;################################################################### LIBTC.INC
; Purpose.....:	Library test program, combined hardware & genp functions
;		To be included by LIBT.MAC
; History.....: 21.02.2023: Created
;#############################################################################

;====================================================================== DFRS()
; Purpose.....:	Send a file to SIO-B in interrupt mode. Used to test ints
;		in combination with disk accesses. In addition, timer ints
;		are also running.
; Requires....: hKR(), hIS(), hCTAS(), hCTAC(), hCTAR(), hCTAB(), hSBS(),
;		gCOSN(), gCOSCN(), gCIC(), gDFCS(), gDFR(), gDFW()
; Labels......: DFRSxx: D)isk F)ile R)ead S)end
;=============================================================================
DFRS::		LD	DE,tDFRSS
		CALL	gCOSCN		; print sign-on message, cursor off 

		SCF
		CALL	hIS		; setup interrupt table and Z80

		LD	A,hcCT10	; 10mS interval
		SCF			; tell hCTAS to initialize
		CALL	hCTAS		; setup timer

;-------------------------------------------------------- setup timer callback
		LD	HL,DFRSTC	; address of callback function
		LD	DE,-1		; run infinite
		LD	BC,50		; interval half a second
		CALL	hCTAC		; setup callback

		SCF
		CALL	hCTAR		; run the timer

;------------------------------------------------------------------- setup SIO
		LD	A,hcSFHD 	; DTR handshake
		LD	B,hcSR9K	; baudrate	
		SCF			; do init
		CALL	hSBS		; init SIO-B

 XOR A
 LD (0FA6AH),A

		EI			; run the interrupts
	
;------------------------------------------------------------- setup operation
		LD	IX,vDFRSF	; IX= flags address
		XOR	A		; clear all flags
		SET	bDFRSR,A	; except read required
		LD	(IX+0),A	; store the flags

		LD	HL,0
		LD	(vDFRSC),HL	; clear sent bytes counter

;----------------------------------------------------------- setup file buffer
cDFRSZ		EQU	0800H		; buffer/file size

cDFRSL		EQU	07EH-021H+2	; length of one pattern string
cDFRSC		EQU	cDFRSZ/cDFRSL	; number of strings in buffer

		LD	HL,(vBUF)	; HL= buffer address
		LD	E,cDFRSC	; B= strings counter

DFRSBS:		LD	C,21H		; start with '!'
		LD	B,cDFRSL-2	; char counter

DFRSB1:		LD	(HL),C		; write to buffer
		INC	HL		; point to next
		INC	C		; next pattern
		DJNZ	DFRSB1		; loop one string

		LD	(HL),gcCR	; terminate string
		INC	HL
		LD	(HL),gcLF
		INC	HL
		DEC	E		; dec string counter
		JR	NZ,DFRSBS	; write next

		LD	(HL),gcEOF	; done, write end-of-file char

;-------------------------------------------------------- write temporary file
DFRSFC::	LD	DE,tDFRSC	; creating...
		CALL	gCOSCN		; display with code

		LD	DE,gcFCB	; DE= FCB address
		LD	BC,tDFRSN	; BC= filename
		XOR	A		; use default drive
		CALL	gDFCS		; setup FCB
	
		LD	HL,(vBUF)	; HL= start address
		LD	DE,cDFRSZ	; DE= file size	
		ADD	HL,DE
		LD	B,H		; BC= end address
		LD	C,L
		LD	HL,(vBUF)	; HL= start address
		LD	DE,gcFCB	; DE= FCB address
		XOR	A		; CY=0: create/overwrite
		CALL	gDFW
		JR	NC,DFRSL	; success, continue

		LD	DE,tDFREW	; error, display message
		JP	DFRSE		; error exit

;------------------------------------------------------------------------ loop
DFRSL:		CALL	hKR		; read keyboard
		CP	gcESC		; ESC?
		JP	Z,DFRSX		; yes, terminate

		CP	gcSPC		; space?	
		JR	NZ,DFRST	; no, serve timer callback

		LD	A,(IX+0)	; yes, toggle display
		XOR	fDFRSF		; to get full speed mode
		LD	(IX+0),A	; save mode
		RES	bDFRSS,(IX+0)	; redisplay sending...

DFRST:		CALL	hCTAB		; timer callback handler

;--------------------------------------------------- read file, 256 byte junks
DFRSR::		BIT	bDFRSR,(IX+0)	; is read required?
		JR	Z,DFRSS		; no, go to send

		RES	bDFRSS,(IX+0)	; reset sending flag
		RES	bDFRSR,(IX+0)	; reset reading request flag
		LD	DE,tDFRSR	; reading...
		CALL	gCOSCN		; display with code

		LD	HL,(vBUF)	; HL= from address
		LD	BC,(vBUF)	; BC= from address
		INC	B		; +256 = to address
		LD	DE,gcFCB	; DE= FCB address
		CALL	gDFR		; read next junk	
		LD	HL,(vBUF)	; HL= buffer start
		JR	NC,DFRSS	; no error, go to send
		LD	DE,tDFRER	; read error
		JR	DFRSE		; error exit

;------------------------------------------------------------------- send data
DFRSS::		LD	A,(HL)		; get next byte to send

		CP	gcEOF		; end of file?
		JR	NZ,DFRSW	; no, write to SIO buffer

		LD	DE,gcFCB	; DE= FCB address
		CALL	gDFCL		; yes, close file
		SET	bDFRSR,(IX+0)	; and set file read request
		JR	DFRSL		; back to loop

DFRSW::		CALL	hSBIW		; write to interrupt buffer
		JR	C,DFRSL		; buffer full, retry same character
		INC	HL		; bump pointer

		BIT	bDFRSS,(IX+0)	; sending displayed?
		JR	NZ,DFRSWC	; yes, display counter
		SET	bDFRSS,(IX+0)	; no, set to displayed
		LD	DE,tDFRST	; sending...
		CALL	gCOSCN		; display with code

DFRSWC:		LD	DE,(vDFRSC)	; get sent bytes counter
		INC	DE		; bump it
		LD	(vDFRSC),DE	; and write back

		BIT	bDFRSF,(IX+0)	; full speed enabled?
		JR	NZ,DFRSL	; yes, back to loop

		PUSH	HL		; save buffer read pointer
		LD	HL,vDFRST	; where to store convert result
		LD	C,5		; number of digits
		XOR	A		; leading blanks
		CALL	gOBD		; convert to ASCII
		LD	DE,vDFRST	; and print it
		LD	BC,cDFRSP	; BC= cursor position
		SCF			; position cursor
		DI			; CP inaccurate if ints enabled
		CALL	gCOS		; display bytes counter
		EI
		POP	HL		; recall buffer read pointer
		JP	DFRSL		; and back to loop

;------------------------------------------------------------------ error exit
DFRSE:		CALL	gCOSN		; error exit, display message
		CALL	gCIC		; wait for key

;--------------------------------------------- termination, de-init interrupts
DFRSX:		XOR	A		; tell hCTAR to stop
		CALL	hCTAR		; stop timer in breakpoint

DFRSB::		NOP			; space for breakpoint
		JR	DFRSQ		; to skip if continue from breakpoint
		SCF			; resume timer
		CALL	hCTAR		; on continue from breakpoint
		JP	DFRSL		; back to loop

DFRSQ:		LD	DE,tDFRSD	; deleting...
		CALL	gCOSCN		; display with code
		LD	DE,gcFCB	; DE= FCB address
		CALL	gDFD		; delete file

		XOR	A		; tell hCTAR to stop
		CALL	hCTAR		; stop timer

		XOR	A		; tell hSBS to de-initialize
		CALL	hSBS		; de-init SIO-A

		XOR	A		; tell hIS to de-initialize		
		CALL	hIS		; terminate interrupt operation

		XOR	A		; redisplay menu / LED off
		OUT	(hcKPLD),A	; capslock LED off

		LD	DE,hvVCON	; turn cursor on
		CALL	gCOD		; send code
		RET

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ DFRSTC()
; Purpose.....:	Timer callback, toggles CAPS-LOCK LED
; Input.......:	All registers undefined
; Labels......: DFRSxx: D)isk F)ile R)ead S)end T)imer C)allback
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
DFRSTC::	LD	A,(vDFRSL)	; get LED status
		XOR	01		; toggle bit0
		LD	(vDFRSL),A	; and write back

		OUT	(hcKPLD),A	; toggle LED
		RET

;------------------------------------------------------------------- variables
vDFRSL:		DB	00		; caps-lock LED status	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ eoF DFRSTC()

;------------------------------------------------------------------- constants
tDFRSN:		DB	'LIBT    TMP'

tDFRSS:		DW	hvVCOF
		DB	gcCR,gcLF
		DB	'Send file to SIO-B in int mode, '
		DB	'<SP> toggles sent bytes display'
		DB	gcCR,gcLF,gcLF,'$'

tDFRSC:		DW	hvVRCL
		DB	'Creating file ...$'

tDFRSD:		DW	hvVRCL
		DB	'Deleting file ...$'

tDFRSR:		DW	hvVRCL
		DB	'Reading file ...$'

tDFRST:		DW	hvVRCL
		DB	'Sending ...  $'
vDFRST:		DB	'      $'

cDFRSP		EQU	0308H
	
tDFREW:		DB	'Disk write error$'
tDFRER:		DB	'Disk read error$'

;------------------------------------------------------------------- variables
bDFRSR		EQU	0		; flag bit0: read file required
bDFRSS		EQU	1		; flag bit1: sending displayed
bDFRSF		EQU	2		; flag bit2: full speed mode
fDFRSF		EQU	04H		; value bit2: full speed mode

vDFRSF:		DB	0		; flags
vDFRSC:		DW	0		; sent bytes counter
;================================================================== eoF DFRS()

;############################################################### EOF LIBTC.INC
================================ eoF DFRS()

;############################################################### EOF LIBTC.INC