;################################################################### LIBTG.MAC
; Purpose.....:	Library test program, general purpose functions
;		To be included by LIBT.MAC
; History.....: 18.10.2023: Created
;#############################################################################

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ utility functions
;====================================================================== UDLY()
; Purpose.....:	Delay specified number of milliseconds
;		Time can be measured on PIO1-A, bit0
; Requires....: hP1
; Labels......: UDLYxx: U)tility D)eL)aY)
;=============================================================================
UDLY::		LD	DE,tUDLYS	; print sign-on message
		CALL	gCOSN

		LD	A,hcPMOU	; PIO output mode
		CALL	hP1AS		; init PIO1, channel A
		
		LD	BC,10		; starting delay in mS
		JR	UDLYS		; display it

;------------------------------------------------------------------ delay loop
UDLYL:		LD	A,1		; set PIO bit0
		SCF			; write outputs
		CALL	hP1AIO		; set bit0 (20uS)
		CALL	gUDLY		; delay BC milliseconds
		XOR	A		; clear PIO bit0
		SCF
		CALL	hP1AIO

		CALL	hKR		; read keyboard
		CP	gcESC		; abort request?
		JR	Z,UDLYX		; yes, goto breakpoint

UDLYP:		CP	'+'		; increase delay by one
		JR	NZ,UDLYM
		INC	BC
		JR	UDLYS		; show new delay

UDLYM:		CP	'-'		; decrease delay by one
		JR	NZ,UDLYZ
		LD	A,B		; prevent overflow to FFFF
		OR	C
		JR	Z,UDLYL		; is zero, loop
		DEC	BC
		JR	UDLYS		; show new delay

UDLYZ:		CP	'0'		; no delay
		JR	NZ,UDLY1
		LD	BC,0
		JR	UDLYS		; show new delay

UDLY1:		CP	'1'		; add 100mS 
		JR	NZ,UDLYL
		LD	HL,100
		ADD	HL,BC
		LD	B,H
		LD	C,L

;----------------------------------------------------------- show actual delay
UDLYS:		PUSH	BC

		LD	A,B		; A= delay hibyte
		CP	3		; max value = 1000
		JR	C,UDLYSP	; B<3, no lobyte check
		JR	NZ,UDLYSR	; B>3, reset
		LD	A,C		; A= delay lobyte
		CP	0E9H		; max is 03E8
		JR	C,UDLYSP	; smaller or equal, no reset

UDLYSR:		POP	BC
		LD	BC,10		; >1000, reset to 10
		PUSH	BC
		
UDLYSP:		LD	D,B
		LD	E,C		; DE= value to convert to decimal
		LD	HL,tUDLYN	; result store address
		LD	C,4		; number of digits to convert
		XOR	A		; no leading zeros
		CALL	gOBD		; convert binary to dec string
		LD	DE,tUDLYD	; HL= address of string
		CALL	gCOSN		; print string
		POP	BC
		JR	UDLYL

;------------------------------------------------------------- breakpoint/exit
UDLYX:		NOP			; space for breakpoint
		JR	UDLYQ		; to skip if continue from breakpoint
		JR	UDLYL		; and continue from breakpoint

UDLYQ:		XOR	A		; redisplay menu
		RET

;-------------------------------------------------------------------- messages
tUDLYD:		DB	gcCR
		DB	'Delay (mS): '
tUDLYN:		DB	'----$'

tUDLYS:		DB	gcCR,gcLF
		DB	'Soft-Delay, trigger on PIO1-A0 '
		DB	'(+/-:1mS, 0:0mS, 1:+100mS)'
		DB	gcCR,gcLF,gcLF,'$'
;================================================================== eoF UDLY()

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++ converting functions
;====================================================================== OBDH()
; Purpose.....:	Convert binary to decimal and hex and vice versa.
;		'-' decreases number of digits, '+' increases them, 
;		'*' adds 1000 to counter
;		'0' toggles leading zeros (decimals only)
;=============================================================================
OBDH::		LD	DE,tOBDHS	; print sign-on message
		CALL	gCOSN

		LD	DE,hvVCOF	; cursor off
		XOR	A		; not embedded in string
		CALL	gCOD		; print code	

		LD	DE,0		; value to convert
		LD	C,5		; initial number of digits
		LD	B,1		; leading zeros flag

OBDHL:		CALL	hKR		; read keyboard
		JR	Z,OBDHD		; no key pressed
		CP	gcESC		; abort request?
		JR	Z,OBDHX		; yes, goto breakpoint

		CP	'*'		; add 1000 to counter
		JR	NZ,OBDHI	; for faster testing hi values
		EX	DE,HL
		LD	DE,1000
		ADD	HL,DE
		EX	DE,HL

OBDHI:		CP	'+'		; increase number of digits
		JR	NZ,OBDHM
		INC	C
		LD	A,C
		CP	6		; 5 is max
		JR	NZ,OBDHD
		DEC	C		; stay on 5 digits

OBDHM:		CP	'-'		; decrease number of digits
		JR	NZ,OBDHZ
		LD	A,C
		CP	1		; 1 is min
		JR	Z,OBDHD
		DEC	C

		PUSH	BC
		PUSH	AF
		LD	A,'$'		; A= terminator
		LD	HL,vOBDR	; HL= start of results buffer
		LD	BC,cOBDHL	; BC= length of results buffer
		CALL	gUMF		; fill with terminators
		POP	AF
		POP	BC

OBDHZ:		CP	'0'		; toggle leading zeros
		JR	NZ,OBDHD
		LD	A,B		; toggle flag
		XOR	01
		LD	B,A

OBDHD:		PUSH	BC
		LD	BC,50		; a short delay to
		CALL	gUDLY		; make result readable
		POP	BC

		PUSH	BC
		RR	B		; B0 to carry, leading zeros
		LD	HL,vOBDR	; HL= result address
		CALL	gOBD		; do bin>dec conversion
		LD	HL,vOBHR
		CALL	gOBH		; do bin>hex conversion
		POP	BC

		PUSH	DE
		LD	DE,vOBDR	; DE= bin>dec result
		CALL	gODB		; convert back to binary
		EX	DE,HL		; DE= binary value
		LD	HL,vOBDC	; HL= dec>bin>dec result
		PUSH	BC
		RR	B		; CY= leading zeros
		CALL	gOBD		; convert to ASCII again
		POP	BC

		LD	DE,vOBHR	; DE= bin>hex result
		CALL	gOHB		; convert to binary
		EX	DE,HL		; DE= binary value
		LD	HL,vOBHC	; HL= hex>bin>hex result
		CALL	gOBH		; convert to ASCII again

		CALL	OBDHP		; print results
		POP	DE

		INC	DE		; increment value
		JP	OBDHL		; restart loop

;------------------------------------------------------------- breakpoint/exit
OBDHX:		NOP			; space for breakpoint
		JR	OBDHQ		; to skip if continue from breakpoint
		JP	OBDHL		; and continue from breakpoint

OBDHQ:		LD	DE,hvVCON	; cursor on
		XOR	A		; not embedded in string
		CALL	gCOD		; print code	
		XOR	A		; redisplay menu
		RET

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ OBDHP()
; Purpose.....:	Print result strings, append spaces in case number of digits
;		changed to less
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
OBDHP:		PUSH	BC
		PUSH	DE

		LD	B,cOBDHR	; starting cursor position, row
		LD	C,cOBDHC	; column	
		LD	HL,vOBDR	; HL= 1st result to display

OBDHPL:		EX	DE,HL		; DE= string address
		SCF			; position cursor to BC
		CALL	gCOS		; print with CP
		EX	DE,HL		; HL= string address
		LD	DE,tOBDHP	; spacer in case less digits
		CALL	gCOSN		; print it
		LD	DE,cOBDHE	; DE= length of one result
		ADD	HL,DE		; HL= next result
		INC	B		; cursor to next row		
		LD	A,cOBDHR+4	; initial + number of results
		CP	B		; reached?
		JR	NZ,OBDHPL	; no, next

		POP	DE
		POP	BC
		RET
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ eoF OBDHP()

;------------------------------------------------------------------- variables
vOBDR:		DB	'$$$$$$'	; bin>dec result buffer
vOBDC:		DB	'$$$$$$'	; dec>bin>dec result buffer
vOBHR:		DB	'$$$$$$'	; bin>hex result buffer
vOBHC:		DB	'$$$$$$'	; hex>bin>hex result buffer

cOBDHL		EQU	$-vOBDR		; length of all results
cOBDHE		EQU	6		; length of one result

tOBDHP:		DB	'    $'		; spacer at line end

;-------------------------------------------------------------------- messages
tOBDHS:		DB	gcCR,gcLF
		DB	'Binary <-> Dec/Hex convert '
		DB	'(+/-:Digits, 0:Leading zeros, *:Add 1000)'
		DB	gcCR,gcLF,gcLF

		DB	'Bin>Dec....: ', gcCR,gcLF
		DB	'Dec>Bin>Dec: ', gcCR,gcLF
		DB	'Bin>Hex....: ', gcCR,gcLF
		DB	'Hex>Bin>Hex: ', '$'

cOBDHR		EQU	3		; initial cursor position
cOBDHC		EQU	13
;================================================================== eoF OBDH()

;++++++++++++++++++++++++++++++++++++++++++++++++++++ keyboard input functions
;======================================================================= CIC()
; Purpose.....:	Keyboard input, single character
; Requires....:	gCOSN(), gCOCP(), gCIC()
; Labels......: CICxxx: C)onsole I)nput C)haracter
;=============================================================================
CIC::		LD	DE,tCICS
		CALL	gCOSN		; print sign-on message

CICRS:		LD	B,cCICR		; input row
		LD	C,cCICC		; input column

;-------------------------------------------------------- convert to uppercase
CICU:		CALL	gCOCP		; position cursor
		LD	A,'!'		; '!' means uppercase
		CALL	gCIC
		CP	gcESC		; abort?
		JR	Z,CICX		; yes
		CP	gcCR		; enter?
		JR	NZ,CICU		; no, stay in mode 
		INC	B		; go to next row

;---------------------------------------------------------------- numeric only
CICN:		CALL	gCOCP		; position cursor
		LD	A,'#'		; '#' means numeric only
		CALL	gCIC
		CP	gcESC		; abort?
		JR	Z,CICX		; yes
		CP	gcCR		; enter?
		JR	NZ,CICN		; no, stay in mode
		INC	B		; go to next row

;------------------------------------------------------------- HEX digits only
CICH:		CALL	gCOCP		; position cursor
		LD	A,'$'		; '$' is HEX only, implies uppercase
		CALL	gCIC
		CP	gcESC		; abort?
		JR	Z,CICX		; yes
		CP	gcCR		; enter?
		JR	NZ,CICH		; no, stay in mode
		INC	B		; go to next row

;----------------------------------------------------------------- Yes/No only
CICY:		CALL	gCOCP		; position cursor
		LD	A,'Y'		; 'Y' is yes/no, implies uppercase
		CALL	gCIC
		CP	gcESC		; abort?
		JR	Z,CICX		; yes
		CP	gcCR		; enter?
		JR	NZ,CICY		; no, stay in mode
		INC	B		; go to next row

;------------------------------------------------------------------ everything
CICE:		CALL	gCOCP		; position cursor
		XOR	A		; 0 means everythig
		CALL	gCIC
		CP	gcESC		; abort?
		JR	Z,CICX		; yes
		CP	gcCR		; enter?
		JR	NZ,CICE		; no, stay in mode
		JR	CICRS		; yes, restart

;----------------------------------------------------------------- termination
CICX:		NOP			; space for breakpoint
		JR	CICQ		; to skip if continue from breakpoint
		JR	CICU		; and continue from breakpoint

CICQ:		XOR	A		; redisplay menu
		RET

;------------------------------------------------------------- sign-on message
tCICS:		DB	gcCR,gcLF
		DB	'Keyboard input, character'
		DB	gcCR,gcLF,gcLF

		DB	'Uppercase.: ', gcCR,gcLF
		DB	'Numeric...: ', gcCR,gcLF
		DB	'Hex.......: ', gcCR,gcLF
		DB	'Yes/No....: ', gcCR,gcLF
		DB	'Everything: ', '$'

cCICR		EQU	3		; initial cursor position
cCICC		EQU	12
;=================================================================== eoF CIC()

;======================================================================= CIS()
; Purpose.....:	Keyboard input, string
; Requires....:	gSTER(), gCOSN(), gCIS(), gODB(), gOBD(), gOHB(), gOBH(),
;		gCOS()
; Labels......: CISxxx: C)onsole I)nput S)tring
;=============================================================================
CIS::		LD	DE,tCISS
		CALL	gCOSN		; print sign-on message

		XOR	A
		CALL	gSTER		; terminator=0, '$' can be entered

;-------------------------------------------------------------- no conversions
CISA:		LD	DE,tCISA	; string buffer
		LD	L,30		; input length
		LD	B,3		; input row
		LD	C,cCISC		; input column
		LD	A,80H		; no conversion, borders
		CALL	gCIS
		JR	C,CISX		; aborted, exit

;----------------------------------------------------------- uppercase convert
CISU:		LD	DE,tCISU	; string buffer
		LD	L,30		; input length
		LD	B,4		; input row
		LD	C,cCISC		; input column
		LD	A,'!'+80H	; uppercase convert, borders
		CALL	gCIS
		JR	C,CISX		; aborted, exit

;---------------------------------------------------------------- numeric only
CISN:		LD	DE,tCISN	; numeric string buffer
		LD	L,5		; string length
		LD      B,5		; input row
		LD	C,cCISC		; input column
		LD	A,'#'+80H	; numeric input, borders
		CALL	gCIS
		JR	C,CISX		; aborted, exit

		LD	DE,tCISN	; DE= numeric string
		LD	C,5		; number of digits
		CALL	gODB		; convert to binary
		EX	DE,HL		; DE= binary
		LD	HL,vCISNC	; HL= numeric string buffer
		LD	C,5		; number of digits
		XOR	A		; CY=0: leading spaces
		CALL	gOBD		; convert back to decimal
		LD	B,5		; output row
		LD	C,45		; output column
		LD	DE,tCISNC	; string to output
		SCF			; do corsor positioning
		CALL	gCOS		; do it

;-------------------------------------------------------------------- HEX only
CISH:		LD	DE,tCISH	; HEX string buffer
		LD	L,4		; string length
		LD      B,6		; input row
		LD	C,cCISC		; input column
		LD	A,'$'+80H	; HEX input, borders
		CALL	gCIS
		JR	C,CISX		; aborted, exit

		LD	DE,tCISH	; DE= HEX string
		LD	C,4		; number of digits
		CALL	gOHB		; convert to binary
		EX	DE,HL		; DE= binary
		LD	HL,vCISHC	; HL= HEX string buffer
		LD	C,4		; number of digits
		CALL	gOBH		; convert back to HEX
		LD	B,6		; output row
		LD	C,45		; output column
		LD	DE,tCISHC	; string to output
		SCF			; do corsor positioning
		CALL	gCOS		; do it

		JR	CISA		; restart

;----------------------------------------------------------------- termination
CISX:		NOP			; space for breakpoint
		JR	CISQ		; to skip if continue from breakpoint
		JR	CISU		; and continue from breakpoint

CISQ:		LD	A,'$'		; string terminator
		CALL	gSTER		; back to CP/M standard

		XOR	A		; redisplay menu
		RET

;-------------------------------------------------------------- text constants
tCISA:		DB	'No conversions                ', 00H
tCISU:		DB	'ALL UPPERCASE                 ', 00H
tCISN:		DB	'12345', 00H
tCISH:		DB	'ABCD', 00H

tCISNC:		DB	'Converted Dec->Bin->Dec: |'
vCISNC:		DB	'-----|', 00H

tCISHC:		DB	'Converted Hex->Bin->Hex: |'
vCISHC:		DB	'----|', 00H

;------------------------------------------------------------- sign-on message
tCISS:		DB	gcCR,gcLF
		DB	'Keyboard input, string '
		DB	'(^S:left ^D:right ^G:delete ^H:backspace ^V:ins/ovr)'
		DB	gcCR,gcLF,gcLF

		DB	'Regular..:',gcCR,gcLF
		DB	'Uppercase:',gcCR,gcLF
		DB	'Numeric..:',gcCR,gcLF
		DB	'HEX......:','$'

cCISC		EQU	11		; input column after prompts
;=================================================================== eoF CIS()

;++++++++++++++++++++++++++++++++++++++++++++++++++++ console output functions
;====================================================================== COPI()
; Purpose.....:	Show progress indicator
; Requires....:	gUDLY(), gCOPI()
; Labels......: COPIxx: C)onsole O)ut P)rogress I)ndicator
;=============================================================================
COPI::		LD	DE,tCOPIS
		CALL	gCOSN		; print sign-on message

		LD	DE,tCOPIP	; prompt for indicator
		CALL	gCOSN

		LD	DE,hvVCOF	; cursor off
		XOR	A		; not embedded in string
		CALL	gCOD		; print code	

;----------------------------------- check keyboard for abort or break request
COPIL:		CALL	hKR		; read keyboard using BIOS
		CP	gcESC		; abort request?
		JR	Z,COPIX		; yes, terminate

		LD	BC,200		; a short delay to make
		CALL	gUDLY		; output readable

		CALL	gCOPI		; show the indicator
		JR	COPIL		; back to loop

;----------------------------------------------------------------- termination
COPIX:		NOP			; space for breakpoint
		JR	COPIQ		; to skip if continue from breakpoint
		JR	COPIL		; and continue from breakpoint

COPIQ:		LD	DE,hvVCON	; cursor on
		XOR	A		; not embedded in string
		CALL	gCOD		; print code	

		XOR	A		; redisplay menu
		RET

;-------------------------------------------------------------------- messages
tCOPIP:		DB	gcCR
		DB	'Progress  '		
		DB	'$'

tCOPIS:		DB	gcCR,gcLF
		DB	'Progress indicator'
		DB	gcCR,gcLF,gcLF,'$'
;================================================================== eoF COPI()

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ disk functions
;====================================================================== DFRW()
; Purpose.....:	Read/write a disk file
; Input.......:	B0=0: read, B0=1: write 
;		B1=0: create, B1=1: append
;		B2=1: delete file, B0 and B1 are ignored
; Requires....:	gDFE(), gDFR(), gDFW(), gCOSN(), gCOSC(), hKR(), gOBH()
; Labels......: DFRWxxx: D)isk F)ile R)ead W)rite
;=============================================================================
cDFRWZ		EQU	800H		; file size to read/write

DFRW::		LD	IX,vDFRWF	; IX= flags address
		LD	(IX+0),B	; save the flags

		LD	HL,gcFCB	; HL= FCB address (default)
		LD	(vDFRWC),HL	; save it

		LD	HL,(vBUF)	; get buffer address
		LD	(vDFRWR),HL	; store from address
		LD	DE,cDFRWZ	; DE= file size
		ADD	HL,DE
		LD	(vDFRWT),HL	; store to address

;------------------------------------------------------------- sign-on message
		LD	DE,tDFRWD	; delete sign-on
		BIT	bDFRWD,B	; delete?
		JR	NZ,DFRWI	; yes
		LD	DE,tDFRWR	; read-signon
		BIT	bDFRWW,B	; write?
		JR	Z,DFRWI		; no	
		LD	DE,tDFRWW	; yes, write-signon
DFRWI:		CALL	gCOSN		; sign-on message

;---------------------------------------------------- display buffer addresses
		BIT	bDFRWD,(IX+0)	; delete file?
		JR	NZ,DFRWF	; yes, no buffer info
		LD	C,4		; convert 4 digits
		LD	DE,(vDFRWR)	; DE= from address
		LD	HL,tDFRWF	; HL= store address
		CALL	gOBH		; convert to HEX string
		LD	DE,(vDFRWT)	; DE= to address
		LD	HL,tDFRWT	; HL= store address
		CALL	gOBH		; convert to HEX string
		LD	DE,tDFRWA	; print addresses
		CALL	gCOSN

;------------------------------------------------------------------- setup FCB
DFRWF:		LD	DE,(vDFRWC)	; DE= FCB address
		LD	BC,tDFRWN	; BC= filename
		XOR	A		; use default drive
		CALL	gDFCS		; setup FCB

;---------------------------------------------------- fill buffer with pattern
DFRWL:		LD	HL,(vDFRWR)	; HL= from address
		LD	DE,(vDFRWT)	; DE= to address
		LD	B,(IX+0)	; modify pattern due to mode
		XOR	A		; start of pattern

DFRWB:		LD	(HL),A		; write pattern to buffer
		INC	HL		; point to next
		ADD	A,B		; next pattern
		CALL	gUHLDE		; end of area?
		JR	NZ,DFRWB	; no, go for next

;----------------------------------------------------------- dispatch function
		LD	A,(IX+0)	; A= flags
		BIT	bDFRWD,A	; delete?
		JP	NZ,DFRWD	; yes
		BIT	bDFRWW,A	; write?
		JR	NZ,DFRWP	; yes

;--------------------------------------------------------------- read the file
DFRWR:		LD	DE,tDFRWG	; reading...
		CALL	gCOSN		; reading message

		LD	DE,(vDFRWC)	; DE= FCB address
		LD	HL,(vDFRWR)	; HL= from address
		LD	BC,(vDFRWT)	; BC= to address
		CALL	gDFR		; read the file
		CP	gcDFRN		; file not found?
		JR	Z,DFRWEI	; yes
		CP	gcDFRE		; eof?
		JR	Z,DFRWEE	; yes
		LD	DE,tDFRWS	; no, success
		JR	DFRWRM		; print it

DFRWEI:		LD	DE,tDFRWI	; file not found
		JR	DFRWRM

DFRWEE:		LD	DE,tDFRWE	; read error (eof)

DFRWRM:		CALL	gCOSN		; success/error message
		JR	DFRWK		; skip write

;--------------------------------------------------------- define write prompt
DFRWP::		LD	BC,tDFRWN	; BC= address of filename
		XOR	A		; on current drive
		CALL	gDFE		; already there?
		LD	DE,tDFRWC	; creating...
		JR	C,DFRWPW	; file not found, create in any case
		LD	DE,tDFRWB	; appending...
		BIT	bDFRWA,(IX+0)	; append selected?
		JR	NZ,DFRWPW	; yes, print it
		LD	DE,tDFRWO	; overwriting...

DFRWPW:		XOR	A		; no cursor positioning
		CALL	gCOSN		; action message

;-------------------------------------------------------------- write the file 
DFRWW:		LD	HL,(vDFRWR)	; HL= from address
		LD	BC,(vDFRWT)	; BC= to address
		LD	DE,(vDFRWC)	; DE= FCB address
		LD	A,(IX+0)	; A= flags, append is bit1
		SRA	A		; shift bit1 to carry
		SRA	A
		CALL	gDFW		; write the file
		CP	gcDFWD		; out of dir space?
		JR	Z,DFRWWD	; yes
		CP	gcDFWF		; disk full/io error?
		JR	Z,DFRWWU	; yes
		LD	DE,tDFRWS	; success
		JR	DFRWWM		; display message

DFRWWD:		LD	DE,tDFRWP	; out of dir space
		JR	DFRWWM

DFRWWU:		LD	DE,tDFRWU	; disk full

DFRWWM:		CALL	gCOSN		; success/error message
		JR	DFRWK		; skip delete

;----------------------------------------------------------------- delete file
DFRWD:		LD	DE,tDFRWL	; deleting...
		CALL	gCOSN		; deleting message

		LD	DE,(vDFRWC)	; BC= FCB address
		CALL	gDFD		; delete file
		CP	gcDFRN		; file not found?
		LD	DE,tDFRWI	; file not found	
		JR	Z,DFRWDM	; yes
		LD	DE,tDFRWS	; success

DFRWDM:		CALL	gCOSN		; success/error message

;---------------------------------------------- wait for keypress to terminate
DFRWK:		CALL	hKR		; wait for key
		CP	gcESC		; abort request?
		JR	Z,DFRWX		; yes, exit
		CP	gcCR		; repeat operation?
		JR	NZ,DFRWK	; no
		LD	DE,hvVRCL	; yes, clear message line
		XOR	A		; not embedded
		CALL	gCOD
		JP	DFRWL		; repeat operation

;----------------------------------------------------------------- termination
DFRWX:		NOP			; space for breakpoint
		JR	DFRWQ		; to skip if continue from breakpoint
		JP	DFRWL		; and continue from breakpoint

DFRWQ:		LD	DE,(vDFRWC)	; DE= FCB address
		CALL	gDFCL		; close the file

DFRWQ1:		XOR	A		; redisplay menu
		RET

;------------------------------------------------------------------- variables
vDFRWR:		DW	0		; from address
vDFRWT:		DW	0		; to address
vDFRWC:		DW	0		; FCB address

bDFRWW		EQU	0		; bit0: write mode
bDFRWA		EQU	1		; bit1: write-append mode
bDFRWD		EQU	2		; bit2: delete mode
vDFRWF:		DB	0		; flags

;-------------------------------------------------------------------- messages
tDFRWN:		DB	'LIBT    BIN'	; filename in FCB format

tDFRWR:		DB	gcCR,gcLF
		DB	'Read $'

tDFRWW:		DB	gcCR,gcLF
		DB	'Write $'

tDFRWD:		DB	gcCR,gcLF
		DB	'Delete LIBT.BIN'
		DB	gcCR,gcLF,gcLF,'$'

tDFRWA:		DB	'LIBT.BIN,'
		DB	' from '
tDFRWF:		DB	'----'
		DB	' to '
tDFRWT:		DB	'----'
		DB	gcCR,gcLF,gcLF,'$'

tDFRWG:		DB	'Reading ... $'
tDFRWC:		DB	'Creating ... $'
tDFRWO:		DB	'Overwriting ... $'
tDFRWB:		DB	'Appending ... $'
tDFRWL:		DB	'Deleting ... $'
tDFRWS:		DB	'Success $'
tDFRWI:		DB	'File not found $'
tDFRWE:		DB	'EOF or I/O error $'
tDFRWP:		DB	'No directory space $'
tDFRWU:		DB	'Disk full or I/O error $'
;================================================================== eoF DFRW()

;======================================================================= DRW()
; Purpose.....:	Read/write disk sectors using BIOS
;		B=0: read, B=1: write 
; Requires....:	gDRW(), gCOSN(), gCIDR(), gCIC(), hKR(), gOBH()
; Labels......: DRWxxx: D)isk R)ead W)rite
;=============================================================================
DRW::		LD	HL,(vBUF)	; get buffer address
		LD	(vDRWF),HL	; store from address
		LD	DE,cDRWZ	; DE= buffer size
		ADD	HL,DE
		LD	(vDRWT),HL	; store to address

		LD	DE,tDRWR	; default read-signon
		BIT	0,B		; write?
		JR	Z,DRWI		; no	
		LD	DE,tDRWW	; yes, write-signon
DRWI:		CALL	gCOSN		; sign-on message

		LD	C,4		; convert 4 digits
		LD	DE,(vDRWF)	; DE= from address
		LD	HL,tDRWF	; HL= store address
		CALL	gOBH		; convert to HEX string
		LD	DE,(vDRWT)	; DE= to address
		LD	HL,tDRWT	; HL= store address
		CALL	gOBH		; convert to HEX string
		LD	DE,tDRWA	; print addresses
		CALL	gCOSN

;---------------------------------------------------- fill buffer with pattern
DRWB:		LD	HL,(vDRWF)	; HL= from address
		LD	DE,(vDRWT)	; DE= to address
		XOR	A		; start of pattern

DRWF:		LD	(HL),A		; write pattern
		INC	HL		; point to next
		ADD	A,B		; next pattern (if write only)
		CALL	gUHLDE		; end of area?
		JR	NZ,DRWF		; no, go for next

;------------------------------------------------------------ ask drive to use
DRWD:		LD	DE,tDRWQ	; drive prompt	
		CALL	gCOSCN		; print with code, no CP
		CALL	gCIDR		; get drive letter, also selects disk
		JR	C,DRWX		; aborted with <ESC>
		JR	Z,DRWD		; not accepted by BIOS
		SUB	'A'		; make binary
		LD	C,A		; drive to C

;--------------------------------------------------------- confirm disk change
		BIT	0,B		; on write only
		JR	Z,DRWRW
		LD	DE,tDRWC	; insert scratch disk prompt
		CALL	gCOSCN		; print with code, no CP
		XOR	A		; allow all input
		CALL	gCIC		; wait for key
		CP	gcESC		; abort?
		JR	Z,DRWD		; yes, reread

;------------------------------------------------------- read/write the tracks
DRWRW:		LD	DE,tDRWD	; reading...
		BIT	0,B
		JR	Z,DRWG		; read selected
		LD	DE,tDRWG	; writing...
DRWG:		CALL	gCOSCN		; reading/writing message

		PUSH	BC		; save flags and drive
		RR	B		; CY=0: read, CY=1: write
		LD	HL,(vDRWF)	; HL= buffer address
		LD	D,cDRWT		; D= track number
		LD	E,cDRWS		; E= sector number
		LD	B,cDRWSC	; B= number of sectors
		CALL	gDRW		; read/write the sectors
		POP	BC		; recall flags and drive
		CP	gcDRWD		; drive select error?
		JR	Z,DRWEL		; yes
		CP	gcDRWI		; i/o error?
		JR	Z,DRWEI		; yes
		LD	DE,tDRWS	; no, success
		JR	DRWM		; print it

DRWEL:		LD	DE,tDRWL	; drive select error
		JR	DRWM

DRWEI:		LD	DE,tDRWI	; i/o error

DRWM:		CALL	gCOSN		; success/error message

;---------------------------------------------- wait for keypress to terminate
DRWL:		CALL	hKR		; read keyboard using BIOS
		CP	gcESC		; abort request?
		JR	Z,DRWX		; yes, exit
		CP	gcCR		; repeat operation?
		JR	NZ,DRWL		; no
		JP	DRWB		; yes, restart

;----------------------------------------------------------------- termination
DRWX:		NOP			; space for breakpoint
		JR	DRWQ		; to skip if continue from breakpoint
		JP	DRWB		; and continue from breakpoint

DRWQ:		XOR	A		; redisplay menu
		RET

;------------------------------------------------------------------- variables
vDRWF:		DW	0		; buffer start
vDRWT:		DW	0		; buffer end

;-------------------------------------------------------------------- messages
tDRWR:		DB	gcCR,gcLF
		DB	'Read $'

tDRWW:		DB	gcCR,gcLF
		DB	'Write $'

tDRWA:		DB	' track 0 and 1, 64 sectors,'
		DB	' from '
tDRWF:		DB	'----'
		DB	' to '
tDRWT:		DB	'----'
		DB	gcCR,gcLF,gcLF,'$'

tDRWQ:		DW	hvVRCL
		DB	'Which drive? $'

tDRWC:		DW	hvVRCL
		DB	'Insert scratch disk $'

tDRWD:		DW	hvVRCL
		DB	'Reading ... $'

tDRWG:		DW	hvVRCL
		DB	'Writing ... $'

tDRWS:		DB	'Success $'
tDRWI:		DB	'I/O error $'
tDRWL:		DB	'Disk select error $'

;----------------------------------------------------------------- definitions
cDRWT		EQU	0		; starting track
cDRWS		EQU	0		; starting sector
cDRWTN		EQU	2		; number of tracks to read/write
cDRWSC		EQU	cDRWTN*32	; number of sectors to read/write
cDRWZ		EQU	cDRWTN*32*128	; tracks * secs/track * sec size
;=================================================================== eoF DRW()

;############################################################### EOF LIBTG.INC
================================= eoF DR